####################################
# single_pred prediction

toxicity_dataset_names = ['toxcast', 'tox21', 'clintox', 'herg', 'dili', 'skin_reaction', 'ames', 'carcinogens_lagunin', 'ld50_zhu']

adme_dataset_names = ['lipophilicity_astrazeneca',
 'solubility_aqsoldb',
 'hydrationfreeenergy_freesolv',
 'caco2_wang',
 'hia_hou',
 'pgp_broccatelli',
 'f20_edrug3d',
 'f30_edrug3d',
 'bioavailability_ma',
 'vd_edrug3d',
 'vdss_lombardo',
 'cyp2c19_veith',
 'cyp2d6_veith',
 'cyp3a4_veith',
 'cyp1a2_veith',
 'cyp2c9_veith',
 'cyp2c9_substrate_carbonmangels', 
 'cyp2d6_substrate_carbonmangels',
 'cyp3a4_substrate_carbonmangels', 
 'halflife_edrug3d',
 'clearance_edrug3d',
 'bbb_adenot',
 'bbb_martins',
 'ppbr_ma',
 'ppbr_edrug3d',
 'ppbr_az',
 'half_life_obach',
 'clearance_hepatocyte_az',
 'clearance_microsome_az']

hts_dataset_names = ['hiv', 
'sarscov2_3clpro_diamond', 
'sarscov2_vitro_touret']

qm_dataset_names = ['qm7b', 'qm8', 'qm9']

epitope_dataset_names = ['iedb_jespersen', 'pdb_jespersen']

paratope_dataset_names = ['sabdab_liberis']

develop_dataset_names = ['tap', 'sabdab_chen']

####################################
# multi_pred prediction

dti_dataset_names = ['davis',
 'kiba',
 'bindingdb_kd',
 'bindingdb_ic50',
 'bindingdb_ki']

ppi_dataset_names = ['huri']

peptidemhc_dataset_names = ['mhc2_iedb_jensen', 'mhc1_iedb-imgt_nielsen']

ddi_dataset_names = ['drugbank', 'twosides']

mti_dataset_names = ['mirtarbase']

gda_dataset_names = ['disgenet']

crisproutcome_dataset_names = ['leenay']

drugres_dataset_names = ['gdsc1', 'gdsc2']

drugsyn_dataset_names = ['oncopolypharmacology', 'drugcomb']

antibodyaff_dataset_names = ['protein_sabdab']

yield_dataset_names = ['uspto_yields', 'buchwald-hartwig']

catalyst_dataset_names = ['uspto_catalyst']


####################################
# generation


retrosyn_dataset_names = ['uspto50k', 'uspto']

forwardsyn_dataset_names = ['uspto']

single_molecule_dataset_names = ['zinc', 'moses', 'chembl']

paired_dataset_names = ['uspto50k', 'uspto']


####################################
# resource

compound_library_names = ['drugbank_drugs', 'chembl_drugs', 'broad_repurposing_hub', 'antivirals']
biokg_library_names = ['hetionet']

####################################
# oracles

#### evaluator for distribution learning, the input of __call__ is list of smiles
distribution_oracles = ['novelty', 'diversity', 'uniqueness', 'validity', 'fcd_distance', 'kl_divergence']  


property_names = ['drd2', 'qed', 'logp', 'sa', 'gsk3b', 'jnk3',]

evaluator_name = ['roc-auc', 'f1', 'pr-auc', 'precision', 'recall', \
				  'accuracy', 'mse', 'rmse', 'mae', 'r2', 'micro-f1', 'macro-f1', \
				  'kappa', 'avg-roc-auc', 'rp@k', 'pr@k', 'pcc', 'spearman']

evaluator_name.extend(distribution_oracles)

guacamol_oracle = ['rediscovery', 'similarity', 'median', 'isomers', 'mpo', 'hop', \
				   'celecoxib_rediscovery', 'troglitazone_rediscovery', 'thiothixene_rediscovery', \
				   'aripiprazole_similarity', 'albuterol_similarity', 'mestranol_similarity', 
				   'isomers_c7h8n2o2', 'isomers_c9h10n2o2pf2cl', \
				   'osimertinib_mpo', 'fexofenadine_mpo', 'ranolazine_mpo', 'perindopril_mpo', \
				   'amlodipine_mpo', 'sitagliptin_mpo', 'zaleplon_mpo', \
				   'median1', 'median2', \
				   'valsartan_smarts', 'deco_hop', 'scaffold_hop']


####################################
# Benchmark Datasets

admet_benchmark = {'ADME': ['caco2_wang', 
							'hia_hou',
							'pgp_broccatelli', 
							'bioavailability_ma',
							'lipophilicity_astrazeneca',
							'solubility_aqsoldb',
							'bbb_martins',
							'ppbr_az',
							'vdss_lombardo',
							'cyp2d6_veith',
							'cyp3a4_veith',
							'cyp2c9_veith',
							'cyp2d6_substrate_carbonmangels',
							'cyp3a4_substrate_carbonmangels',
							'cyp2c9_substrate_carbonmangels',
							'half_life_obach',
							'clearance_microsome_az',
							'clearance_hepatocyte_az'],
					'Tox':['herg',
							'ames',
							'dili',
							'ld50_zhu']
					}

drugsyn_benchmark = {'Synergy': ['drugcomb_css',
                                 'drugcomb_hsa',
                                 'drugcomb_loewe',
                                 'drugcomb_bliss',
                                 'drugcomb_zip'
                                 ]}

####################################

#### Benchmark Metrics
admet_metrics = {'caco2_wang': 'mae',
				'hia_hou': 'roc-auc',
				'pgp_broccatelli': 'roc-auc', 
				'bioavailability_ma': 'roc-auc',
				'lipophilicity_astrazeneca': 'mae',
				'solubility_aqsoldb': 'mae',
				'bbb_martins': 'roc-auc',
				'ppbr_az': 'mae',
				'vdss_lombardo': 'spearman',
				'cyp2c9_veith': 'pr-auc',
				'cyp2d6_veith': 'pr-auc',
				'cyp3a4_veith': 'pr-auc',
				'cyp2c9_substrate_carbonmangels': 'pr-auc',
				'cyp3a4_substrate_carbonmangels': 'roc-auc',
				'cyp2d6_substrate_carbonmangels': 'pr-auc',
				'half_life_obach': 'spearman',
				'clearance_hepatocyte_az': 'spearman',
				'clearance_microsome_az': 'spearman',
				'ld50_zhu': 'mae',
				'herg': 'roc-auc',
				'ames': 'roc-auc',
				'dili': 'roc-auc'
				}

drugsyn_metrics = {'drugcomb_css': 'mae',
                   'drugcomb_hsa':'mae',
                  'drugcomb_loewe':'mae',
                  'drugcomb_bliss':'mae',
                  'drugcomb_zip':'mae',
                  'drugcomb_css_brain':'mae',
                  'drugcomb_css_ovary':'mae',
                  'drugcomb_css_lung':'mae',
                  'drugcomb_css_skin':'mae',
                  'drugcomb_css_hematopoietic_lymphoid':'mae',
                  'drugcomb_css_breast':'mae',
                  'drugcomb_css_prostate':'mae',
                  'drugcomb_css_kidney':'mae',
                  'drugcomb_css_colon':'mae',
                   }

#### Benchmark Splits
admet_splits = {'caco2_wang': 'scaffold',
				'hia_hou': 'scaffold',
				'pgp_broccatelli': 'scaffold', 
				'bioavailability_ma': 'scaffold',
				'lipophilicity_astrazeneca': 'scaffold',
				'solubility_aqsoldb': 'scaffold',
				'bbb_martins': 'scaffold',
				'ppbr_az': 'scaffold',
				'vdss_lombardo': 'scaffold',
				'cyp2c9_veith': 'scaffold',
				'cyp2d6_veith': 'scaffold',
				'cyp3a4_veith': 'scaffold',
				'cyp2c9_substrate_carbonmangels': 'scaffold',
				'cyp3a4_substrate_carbonmangels': 'scaffold',
				'cyp2d6_substrate_carbonmangels': 'scaffold',
				'half_life_obach': 'scaffold',
				'clearance_hepatocyte_az': 'scaffold',
				'clearance_microsome_az': 'scaffold',
				'ld50_zhu': 'scaffold',
				'herg': 'scaffold',
				'ames': 'scaffold',
				'dili': 'scaffold'
				}

drugsyn_splits = {'drugcomb_css': 'combination',
                    'drugcomb_hsa': 'combination',
                    'drugcomb_loewe': 'combination',
                    'drugcomb_bliss': 'combination',
                    'drugcomb_zip': 'combination'
                    }

####################################

# evaluator for single molecule, the input of __call__ is a single smiles OR list of smiles
download_oracle_names = ['drd2', 'gsk3b', 'jnk3', 'fpscores', 'cyp3a4_veith']
trivial_oracle_names = ['qed', 'logp', 'sa'] + guacamol_oracle
synthetic_oracle_name = ['askcos', 'ibm_rxn']

meta_oracle_name = ['isomer_meta', 'rediscovery_meta', 'similarity_meta', 'median_meta', 'docking_score', 'molecule_one_synthesis']

oracle_names = download_oracle_names + trivial_oracle_names + distribution_oracles + synthetic_oracle_name + meta_oracle_name 

molgenpaired_dataset_names = ['qed', 'drd2', 'logp']

generation_datasets = retrosyn_dataset_names + forwardsyn_dataset_names + molgenpaired_dataset_names 
# generation
####################################

category_names = {'single_pred': ["Tox",
									"ADME",
									"HTS",
									"Epitope",
									"Develop",
									"QM",
									"Paratope",
									"Yields",
									"CRISPROutcome"],
				'multi_pred': ["DTI",
								"PPI",
								"DDI",
								"PeptideMHC",
								"DrugRes",
								"AntibodyAff",
								"DrugSyn",
								"MTI",
								"GDA",
								"Catalyst"],
				'generation': ["RetroSyn",
								"Reaction",
								"PairMolGen",
								"MolGen"
								]
				}

def get_task2category():
	task2category = {}
	for i, j in category_names.items():
		for x in j:
			task2category[x] = i
	return task2category

dataset_names = {"Toxicity": toxicity_dataset_names, 
				"ADME": adme_dataset_names, 
				"HTS": hts_dataset_names, 
				"DTI": dti_dataset_names, 
				"PPI": ppi_dataset_names, 
				"DDI": ddi_dataset_names,
				"RetroSyn": retrosyn_dataset_names,
				"Reaction": forwardsyn_dataset_names, 
				"PairMolGen": molgenpaired_dataset_names,
				"MolGen": single_molecule_dataset_names,
				"PeptideMHC": peptidemhc_dataset_names,
				"Epitope": epitope_dataset_names,
				"Develop": develop_dataset_names,
				"DrugRes": drugres_dataset_names,
				"QM": qm_dataset_names,
				"AntibodyAff": antibodyaff_dataset_names,
				"DrugSyn": drugsyn_dataset_names,
				"MTI": mti_dataset_names,
				"GDA": gda_dataset_names,
				"Paratope": paratope_dataset_names, 
				"Yields": yield_dataset_names, 
				"Catalyst": catalyst_dataset_names, 
				"CompoundLibrary": compound_library_names,
				"BioKG": biokg_library_names,
				"CRISPROutcome": crisproutcome_dataset_names
				}

benchmark_names = {"admet_group": admet_benchmark,
                   "drugcombo_group": drugsyn_benchmark}
bm_metric_names = {"admet_group": admet_metrics,
                  "drugcombo_group": drugsyn_metrics}
bm_split_names = {"admet_group": admet_splits,
                  "drugcombo_group": drugsyn_splits}

dataset_list = []
for i in dataset_names.keys():
    dataset_list = dataset_list + [i.lower() for i in dataset_names[i]]

name2type = {'toxcast': 'tab',
 'tox21': 'tab',
 'clintox': 'tab',
 'lipophilicity_astrazeneca': 'tab',
 'solubility_aqsoldb': 'tab',
 'hydrationfreeenergy_freesolv': 'tab',
 'caco2_wang': 'tab',
 'hia_hou': 'tab',
 'pgp_broccatelli': 'tab',
 'f20_edrug3d': 'tab',
 'f30_edrug3d': 'tab',
 'bioavailability_ma': 'tab',
 'vd_edrug3d': 'tab',
 'cyp2c19_veith': 'tab',
 'cyp2d6_veith': 'tab',
 'cyp3a4_veith': 'tab',
 'cyp1a2_veith': 'tab',
 'cyp2c9_veith': 'tab',
 'cyp2c9_substrate_carbonmangels': 'tab', 
 'cyp2d6_substrate_carbonmangels': 'tab',
 'cyp3a4_substrate_carbonmangels': 'tab', 
 'carcinogens_lagunin': 'tab', 
 'halflife_edrug3d': 'tab',
 'clearance_edrug3d': 'tab',
 'bbb_adenot': 'tab',
 'bbb_martins': 'tab',
 'ppbr_ma': 'tab',
 'ppbr_edrug3d': 'tab',
 'hiv': 'tab',
 'sarscov2_3clpro_diamond': 'tab',
 'sarscov2_vitro_touret': 'tab',
 'davis': 'tab',
 'kiba': 'tab',
 'bindingdb_kd': 'tab',
 'bindingdb_ic50': 'csv',
 'bindingdb_ki': 'csv',
 'huri': 'tab',
 'drugbank': 'tab',
 'twosides': 'csv',
 'mhc1_iedb-imgt_nielsen': 'tab',
 'mhc2_iedb_jensen': 'tab',
 'uspto': 'csv',
 'uspto50k': 'tab',
 'zinc': 'tab', 
 'moses': 'tab',
 'chembl': 'tab',
 'qed': 'tab', 
 'drd2': 'tab', 
 'logp': 'tab',
 'drugcomb':'pkl',
 'gdsc1': 'pkl',
 'gdsc2': 'pkl',
 'iedb_jespersen': 'pkl',
 'pdb_jespersen': 'pkl',
 'qm7b': 'pkl',
 'qm8': 'pkl',
 'qm9': 'pkl',
 'tap': 'tab',
 'sabdab_chen': 'tab',
 'protein_sabdab': 'csv',
 'oncopolypharmacology': 'pkl',
 'mirtarbase': 'csv',
 'disgenet': 'csv',
 'sabdab_liberis': 'pkl',
 'uspto_yields': 'pkl', 
 'uspto_catalyst': 'csv',
 'buchwald-hartwig': 'pkl',
 'hetionet':'tab', 
 'herg': 'tab',
 'dili': 'tab',
 'ppbr_az': 'tab',
 'ames': 'tab',
 'skin_reaction': 'tab',
 'drugbank_drugs': 'csv', 
 'clearance_microsome_az': 'tab',
 'clearance_hepatocyte_az': 'tab',
 'half_life_obach': 'tab',
 'ld50_zhu': 'tab',
 'vdss_lombardo': 'tab',
 'leenay':'tab'}

name2id = {'bbb_adenot': 4259565,
 'bbb_martins': 4259566,
 'bindingdb_ic50': 4291560,
 'bindingdb_kd': 4291555,
 'bindingdb_ki': 4291556,
 'bioavailability_ma': 4259567,
 'caco2_wang': 4259569,
 'clearance_edrug3d': 4259571,
 'clintox': 4259572,
 'cyp1a2_veith': 4259573,
 'cyp2c19_veith': 4259576,
 'cyp2c9_veith': 4259577,
 'cyp2d6_veith': 4259580,
 'cyp3a4_veith': 4259582,
 'cyp2c9_substrate_carbonmangels': 4259584,
 'cyp2d6_substrate_carbonmangels': 4259578,
 'cyp3a4_substrate_carbonmangels': 4259581, 
 'carcinogens_lagunin': 4259570,
 'davis': 4291557,
 'drugbank': 4139573,
 'drugcomb': 4215720,
 'f20_edrug3d': 4259586,
 'f30_edrug3d': 4259589,
 'halflife_edrug3d': 4259587,
 'hia_hou': 4259591,
 'hiv': 4259593,
 'huri': 4139567,
 'hydrationfreeenergy_freesolv': 4259594,
 'kiba': 4291561,
 'lipophilicity_astrazeneca': 4259595,
 'pgp_broccatelli': 4259597,
 'ppbr_edrug3d': 4259600,
 'ppbr_ma': 4259603,
 'sarscov2_3clpro_diamond': 4259606,
 'sarscov2_vitro_touret': 4259607,
 'solubility_aqsoldb': 4259610,
 'tox21': 4259612,
 'toxcast': 4259613,
 'twosides': 4139574,
 'vd_edrug3d': 4259618,
 'mhc1_iedb-imgt_nielsen': 4167073,
 'mhc2_iedb_jensen': 4167074,
 'zinc': 4170963,
 'moses': 4170962,
 'chembl': 4170965,
 'qed': 4170959, 
 'drd2': 4170957, 
 'logp': 4170961, 
 'gdsc1': 4165726,
 'gdsc2': 4165727,
 'iedb_jespersen': 4165725, 
 'pdb_jespersen': 4165724,
 'qm7b': 4167096,
 'qm8': 4167110,
 'qm9': 4167112,
 'tap': 4167113,
 'sabdab_chen': 4167164,
 'protein_sabdab': 4167357,
 'oncopolypharmacology': 4167358,
 'mirtarbase': 4167359,
 'disgenet': 4168282,
 'sabdab_liberis': 4168425,
 'uspto50k': 4171823,
 'buchwald-hartwig': 4186955,
 'uspto_yields': 4186956, 
 'uspto_catalyst': 4171574,
 'uspto': 4171642, 
 'hetionet': 4201734,
 'herg': 4259588,
 'dili': 4259585,
 'ppbr_az': 4259599,
 'ames': 4259564,
 'skin_reaction': 4259609,
 'clearance_microsome_az': 4266186,
 'clearance_hepatocyte_az': 4266187,
 'ld50_zhu': 4267146,
 'half_life_obach': 4266799,
 'vdss_lombardo': 4267387,
 'leenay':4279966 }

oracle2type = {'drd2': 'pkl', 
			   'jnk3': 'pkl', 
			   'gsk3b': 'pkl',
			   'fpscores': 'pkl', 
			   'cyp3a4_veith': 'pkl', 
			   }

oracle2id = {'drd2': 4178625,
			 'gsk3b': 4170295,
			 'jnk3': 4170293,
			 'fpscores': 4170416, 
			 'cyp3a4_veith': 4411249, 
			}

benchmark2type = {'admet_group': 'zip',
                  'drugcombo_group': 'zip'}

benchmark2id = {'admet_group': 4426004,
                'drugcombo_group':  4426002}

sdf_file_names = {
	'grambow': ['Product', 'Reactant', 'TS']
}



import requests
from zipfile import ZipFile 
import os, sys
import numpy as np
import pandas as pd
from pandas.errors import EmptyDataError
import json
import warnings
warnings.filterwarnings("ignore")
import subprocess
import pickle
from fuzzywuzzy import fuzz
from tqdm import tqdm
from .metadata import name2type, name2id, dataset_list, dataset_names, benchmark_names, benchmark2id, benchmark2type
from .metadata import property_names, paired_dataset_names, single_molecule_dataset_names
from .metadata import retrosyn_dataset_names, forwardsyn_dataset_names, molgenpaired_dataset_names, generation_datasets
from .metadata import oracle2id, download_oracle_names, trivial_oracle_names, oracle_names, oracle2type 

from .label_name_list import dataset2target_lists

try:
	from urllib.error import HTTPError
	from urllib.parse import quote, urlencode
	from urllib.request import urlopen
except ImportError:
	from urllib import urlencode
	from urllib2 import quote, urlopen, HTTPError

def fuzzy_search(name, dataset_names):
	name = name.lower()
	if name[:4] == 'tdc.':
		name = name[4:]
	if name in dataset_names:
		s =  name
	else:
		# print("========fuzzysearch=======", dataset_names, name)
		s =  get_closet_match(dataset_names, name)[0]
	if s in dataset_names:
		return s
	else:
		raise ValueError(s + " does not belong to this task, please refer to the correct task name!")

def download_wrapper(name, path, dataset_names):
	name = fuzzy_search(name, dataset_names)
	server_path = 'https://dataverse.harvard.edu/api/access/datafile/'

	dataset_path = server_path + str(name2id[name])

	if not os.path.exists(path):
		os.mkdir(path)

	if os.path.exists(os.path.join(path, name + '.' + name2type[name])):
		print_sys('Found local copy...')
	else:
		print_sys("Downloading...")
		dataverse_download(dataset_path, path, name, name2type)
	return name

def zip_data_download_wrapper(name, path, dataset_names):
	name = fuzzy_search(name, dataset_names)
	server_path = 'https://dataverse.harvard.edu/api/access/datafile/'

	dataset_path = server_path + str(name2id[name])

	if not os.path.exists(path):
		os.mkdir(path)

	if os.path.exists(os.path.join(path, name)):
		print_sys('Found local copy...')
	else:
		print_sys('Downloading...')
		dataverse_download(dataset_path, path, name, name2type)
		print_sys('Extracting zip file...')
		with ZipFile(os.path.join(path, name + '.zip'), 'r') as zip:
			zip.extractall(path = os.path.join(path))
		print_sys("Done!")
	return name

def oracle_download_wrapper(name, path, oracle_names):
	name = fuzzy_search(name, oracle_names)
	if name in trivial_oracle_names:
		return name

	server_path = 'https://dataverse.harvard.edu/api/access/datafile/'
	dataset_path = server_path + str(oracle2id[name])

	if not os.path.exists(path):
		os.mkdir(path)

	if os.path.exists(os.path.join(path, name + '.' + oracle2type[name])):
		print_sys('Found local copy...')
	else:
		print_sys("Downloading Oracle...")
		dataverse_download(dataset_path, path, name, oracle2type) ## to-do to-check
		print_sys("Done!")
	return name

def bm_download_wrapper(name, path):
	name = fuzzy_search(name, list(benchmark_names.keys()))
	server_path = 'https://dataverse.harvard.edu/api/access/datafile/'
	dataset_path = server_path + str(benchmark2id[name])

	if not os.path.exists(path):
		os.mkdir(path)

	if os.path.exists(os.path.join(path, name)):
		print_sys('Found local copy...')
	else:
		print_sys('Downloading Benchmark Group...')
		dataverse_download(dataset_path, path, name, benchmark2type)
		print_sys('Extracting zip file...')
		with ZipFile(os.path.join(path, name + '.zip'), 'r') as zip:
			zip.extractall(path = os.path.join(path))
		print_sys("Done!")
	return name

def pd_load(name, path):
	try:
		if name2type[name] == 'tab':
			df = pd.read_csv(os.path.join(path, name + '.' + name2type[name]), sep = '\t')
		elif name2type[name] == 'csv':
			df = pd.read_csv(os.path.join(path, name + '.' + name2type[name]))
		elif name2type[name] == 'pkl':
			df = pd.read_pickle(os.path.join(path, name + '.' + name2type[name]))
		elif name2type[name] == 'zip':
			df = pd.read_pickle(os.path.join(path, name + '/' + name + '.pkl'))
		else:
			raise ValueError("The file type must be one of tab/csv/pickle.")
		try:
			df = df.drop_duplicates()
		except:
			pass
		return df
	except (EmptyDataError, EOFError) as e:
		import sys
		sys.exit("TDC is hosted in Harvard Dataverse and it is currently under maintenance, please check back in a few hours or checkout https://dataverse.harvard.edu/.")

def property_dataset_load(name, path, target, dataset_names):
	if target is None:
		target = 'Y'
	name = download_wrapper(name, path, dataset_names)
	print_sys('Loading...')
	df = pd_load(name, path)
	try:
		if target is not None:
			target = fuzzy_search(target, df.columns.values)
		df = df[df[target].notnull()].reset_index(drop = True)
	except:
		with open(os.path.join(path, name + '.' + name2type[name]), 'r') as f:
			flag = 'Service Unavailable' in ' '.join(f.readlines())
			if flag:
				import sys
				sys.exit("TDC is hosted in Harvard Dataverse and it is currently under maintenance, please check back in a few hours or checkout https://dataverse.harvard.edu/.")
			else:
				sys.exit("Please report this error to cosamhkx@gmail.com, thanks!")
	try:
		return df['X'], df[target], df['ID']
	except:
		return df['Drug'], df[target], df['Drug_ID']

def molpair_process(name, path, dataset_names):
	name = download_wrapper(name, path, dataset_names)
	print_sys('Loading...')
	df = pd_load(name, path)
	return df['input'], df['output']

def interaction_dataset_load(name, path, target, dataset_names):
	name = download_wrapper(name, path, dataset_names)
	print_sys('Loading...')
	df = pd_load(name, path)
	try:
		if target is None:
			target = 'Y'
		if target not in df.columns.values:
			# for binary interaction data, the labels are all 1. negative samples can be sampled from utils.NegSample function
			df[target] = 1
		if target is not None:
			target = fuzzy_search(target, df.columns.values)
		df = df[df[target].notnull()].reset_index(drop = True)
		return df['X1'], df['X2'], df[target], df['ID1'], df['ID2']
	except:
		with open(os.path.join(path, name + '.' + name2type[name]), 'r') as f:
			flag = 'Service Unavailable' in ' '.join(f.readlines())
			if flag:
				import sys
				sys.exit("TDC is hosted in Harvard Dataverse and it is currently under maintenance, please check back in a few hours or checkout https://dataverse.harvard.edu/.")
			else:
				sys.exit("Please report this error to cosamhkx@gmail.com, thanks!")


def multi_dataset_load(name, path, dataset_names):
	name = download_wrapper(name, path, dataset_names)
	print_sys('Loading...')
	df = pd_load(name, path)
	return df

def generation_paired_dataset_load(name, path, dataset_names, input_name, output_name):
	name = download_wrapper(name, path, dataset_names)
	print_sys('Loading...')
	df = pd_load(name, path)
	return df[input_name], df[output_name]

def three_dim_dataset_load(name, path, dataset_names):
	name = zip_data_download_wrapper(name, path, dataset_names)
	print_sys('Loading...')
	df = pd_load(name, path)
	return df, os.path.join(path, name), name

def distribution_dataset_load(name, path, dataset_names, column_name):
	name = download_wrapper(name, path, dataset_names)
	print_sys('Loading...')
	df = pd_load(name, path)
	return df[column_name]

def generation_dataset_load(name, path, dataset_names):
	name = download_wrapper(name, path, dataset_names)
	print_sys('Loading...')
	df = pd_load(name, path)
	return df['input'], df['target']

def oracle_load(name, path = './oracle', oracle_names = oracle_names):
	name = oracle_download_wrapper(name, path, oracle_names)
	return name

def bm_group_load(name, path):
	name = bm_download_wrapper(name, path)
	return name

def get_label_map(name, path = './data', target = None, file_format = 'csv', output_format = 'dict', task = 'DDI'):
	name = fuzzy_search(name, dataset_names[task])
	if target is None:
		target = 'Y'
	df = pd_load(name, path)

	if output_format == 'dict':
		return dict(zip(df[target].values, df['Map'].values))
	elif output_format == 'df':
		return df
	elif output_format == 'array':
		return df['Map'].values
	else:
		raise ValueError("Please use the correct output format, select from dict, df, array.")

def get_reaction_type(name, path = './data', output_format = 'array'):
	name = fuzzy_search(name, dataset_names['RetroSyn'])
	df = pd_load(name, path)

	if output_format == 'df':
		return df
	elif output_format == 'array':
		return df['category'].values
	else:
		raise ValueError("Please use the correct output format, select from df, array.")

def dataverse_download(url, path, name, types):
	save_path = os.path.join(path, name + '.' + types[name])
	response = requests.get(url, stream=True)
	total_size_in_bytes= int(response.headers.get('content-length', 0))
	block_size = 1024
	progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)
	with open(save_path, 'wb') as file:
		for data in response.iter_content(block_size):
			progress_bar.update(len(data))
			file.write(data)
	progress_bar.close()

def convert_y_unit(y, from_, to_):
	"""
	Arguments:
		y: a list of labels
		from_: 'nM' or 'p'
		to_: 'nM' or 'p'

	Returns:
		y: a numpy array of transformed labels
	"""
	if from_ == 'nM':
		y = y
	elif from_ == 'p':
		y = (10**(-y) - 1e-10)/ 1e-9

	if to_ == 'p':
		y = -np.log10(y*1e-9 + 1e-10)
	elif to_ == 'nM':
		y = y

	return y

def label_transform(y, binary, threshold, convert_to_log, verbose = True, order = 'descending'):
	"""
	Arguments:
		y: a list of labels
		binary: binarize the label given the threshold
		threshold: threshold values
		convert_to_log: for continuous values such as Kd and etc

	Returns:
		y: a numpy array of transformed labels
	"""

	if (len(np.unique(y)) > 2) and binary:
		if verbose:
			print("Binariztion using threshold' + str(threshold) + ', you use specify your threhsold values by threshold = X)", flush = True, file = sys.stderr)
		if order == 'descending':
			y = np.array([1 if i else 0 for i in np.array(y) < threshold])
		elif order == 'ascending':
			y = np.array([1 if i else 0 for i in np.array(y) > threshold])
		else:
			raise ValueError("Please select order from 'descending or ascending!")
	else:
		if (len(np.unique(y)) > 2) and convert_to_log:
			if verbose:
				print('To log space...', flush = True, file = sys.stderr)
			y = convert_y_unit(np.array(y), 'nM', 'p')
		else:
			y = y

	return y

def convert_to_log(y):
	y = convert_y_unit(np.array(y), 'nM', 'p')
	return y

def convert_back_log(y):
	y = convert_y_unit(np.array(y), 'p', 'nM')
	return y

def binarize(y, threshold, order = 'ascending'):
	if order == 'ascending':
		y = np.array([1 if i else 0 for i in np.array(y) > threshold])
	elif order == 'descending':
		y = np.array([1 if i else 0 for i in np.array(y) < threshold])
	else:
		raise AttributeError("'order' must be either ascending or descending")
	return y

def label_dist(y, name = None):

	try:
		import seaborn as sns
		import matplotlib.pyplot as plt
	except:
		utils.install("seaborn")
		utils.install("matplotlib")
		import seaborn as sns
		import matplotlib.pyplot as plt

	median = np.median(y)
	mean = np.mean(y)

	f, (ax_box, ax_hist) = plt.subplots(2, sharex=True, gridspec_kw= {"height_ratios": (0.15, 1)})

	if name is None:
		sns.boxplot(y, ax=ax_box).set_title("Label Distribution")
	else:
		sns.boxplot(y, ax=ax_box).set_title("Label Distribution of " + str(name) + " Dataset")
	ax_box.axvline(median, color='b', linestyle='--')
	ax_box.axvline(mean, color='g', linestyle='--')

	sns.distplot(y, ax = ax_hist)
	ax_hist.axvline(median, color='b', linestyle='--')
	ax_hist.axvline(mean, color='g', linestyle='--')
	ax_hist.legend({'Median':median,'Mean':mean})

	ax_box.set(xlabel='')
	plt.show()
	#print("The median is " + str(median), flush = True, file = sys.stderr)
	#print("The mean is " + str(mean), flush = True, file = sys.stderr)


# random split
def create_fold(df, fold_seed, frac):
	train_frac, val_frac, test_frac = frac
	test = df.sample(frac = test_frac, replace = False, random_state = fold_seed)
	train_val = df[~df.index.isin(test.index)]
	val = train_val.sample(frac = val_frac/(1-test_frac), replace = False, random_state = 1)
	train = train_val[~train_val.index.isin(val.index)]

	return {'train': train.reset_index(drop = True),
			'valid': val.reset_index(drop = True),
			'test': test.reset_index(drop = True)}

# cold setting
def create_fold_setting_cold(df, fold_seed, frac, entity):
	train_frac, val_frac, test_frac = frac
	gene_drop = df[entity].drop_duplicates().sample(frac = test_frac, replace = False, random_state = fold_seed).values

	test = df[df[entity].isin(gene_drop)]

	train_val = df[~df[entity].isin(gene_drop)]

	gene_drop_val = train_val[entity].drop_duplicates().sample(frac = val_frac/(1-test_frac), replace = False, random_state = fold_seed).values
	val = train_val[train_val[entity].isin(gene_drop_val)]
	train = train_val[~train_val[entity].isin(gene_drop_val)]

	return {'train': train.reset_index(drop = True),
			'valid': val.reset_index(drop = True),
			'test': test.reset_index(drop = True)}

# scaffold split
def create_scaffold_split(df, seed, frac, entity):
	# reference: https://github.com/chemprop/chemprop/blob/master/chemprop/data/scaffold.py
	try:
		from rdkit import Chem
		from rdkit.Chem.Scaffolds import MurckoScaffold
		from rdkit import RDLogger
		RDLogger.DisableLog('rdApp.*')
	except:
		raise ImportError("Please install rdkit by 'conda install -c conda-forge rdkit'! ")
	from tqdm import tqdm
	from random import Random

	from collections import defaultdict
	random = Random(seed)

	s = df[entity].values
	scaffolds = defaultdict(set)
	idx2mol = dict(zip(list(range(len(s))),s))

	error_smiles = 0
	for i, smiles in tqdm(enumerate(s), total=len(s)):
		try:
			scaffold = MurckoScaffold.MurckoScaffoldSmiles(mol = Chem.MolFromSmiles(smiles), includeChirality = False)
			scaffolds[scaffold].add(i)
		except:
			print_sys(smiles + ' returns RDKit error and is thus omitted...')
			error_smiles += 1

	train, val, test = [], [], []
	train_size = int((len(df) - error_smiles) * frac[0])
	val_size = int((len(df) - error_smiles) * frac[1])
	test_size = (len(df) - error_smiles) - train_size - val_size
	train_scaffold_count, val_scaffold_count, test_scaffold_count = 0, 0, 0

	#index_sets = sorted(list(scaffolds.values()), key=lambda i: len(i), reverse=True)
	index_sets = list(scaffolds.values())
	big_index_sets = []
	small_index_sets = []
	for index_set in index_sets:
		if len(index_set) > val_size / 2 or len(index_set) > test_size / 2:
			big_index_sets.append(index_set)
		else:
			small_index_sets.append(index_set)
	random.seed(seed)
	random.shuffle(big_index_sets)
	random.shuffle(small_index_sets)
	index_sets = big_index_sets + small_index_sets

	if frac[2] == 0:
		for index_set in index_sets:
			if len(train) + len(index_set) <= train_size:
				train += index_set
				train_scaffold_count += 1
			else:
				val += index_set
				val_scaffold_count += 1
	else:
		for index_set in index_sets:
			if len(train) + len(index_set) <= train_size:
				train += index_set
				train_scaffold_count += 1
			elif len(val) + len(index_set) <= val_size:
				val += index_set
				val_scaffold_count += 1
			else:
				test += index_set
				test_scaffold_count += 1

	return {'train': df.iloc[train].reset_index(drop = True),
			'valid': df.iloc[val].reset_index(drop = True),
			'test': df.iloc[test].reset_index(drop = True)}

def create_combination_split(df, seed, frac):
	"""
	Function for splitting drug combination dataset such that no
	combinations are shared across the split

	:param df: dataset to split as pd Dataframe
	:param seed: random seed
	:param frac: [train, val, test] split fraction as a list
	:return: dictionary of {train, valid, test} datasets
	"""

	# Set split size
	test_size = int(len(df) * frac[2])
	train_size = int(len(df) * frac[0])
	val_size = len(df) - train_size - test_size
	np.random.seed(seed)

	# Create a new column for combination names
	df['concat'] = df['Drug1_ID'] + ',' + df['Drug2_ID']

	# Identify shared drug combinations across all target classes
	combinations = []
	for c in df['Cell_Line_ID'].unique():
		df_cell = df[df['Cell_Line_ID'] == c]
		combinations.append(set(df_cell['concat'].values))

	intxn = combinations[0]
	for c in combinations:
		intxn = intxn.intersection(c)

	# Split combinations into train, val and test
	test_choices = np.random.choice(list(intxn),
						int(test_size / len(df['Cell_Line_ID'].unique())),
						replace=False)
	trainval_intxn = intxn.difference(test_choices)
	val_choices = np.random.choice(list(trainval_intxn),
						int(val_size / len(df['Cell_Line_ID'].unique())),
						replace=False)

	## Create train and test set
	test_set = df[df['concat'].isin(test_choices)].drop(columns=['concat'])
	val_set = df[df['concat'].isin(val_choices)]
	train_set = df[~df['concat'].isin(test_choices)].reset_index(drop=True)
	train_set = train_set[~train_set['concat'].isin(val_choices)]

	return {'train': train_set.reset_index(drop = True),
			'valid': val_set.reset_index(drop = True),
			'test': test_set.reset_index(drop = True)}

def train_val_test_split(len_data, frac, seed):
	test_size = int(len_data * frac[2])
	train_size = int(len_data * frac[0])
	val_size = len_data - train_size - test_size
	np.random.seed(seed)
	x = np.array(list(range(len_data)))
	np.random.shuffle(x)
	return x[:train_size], x[train_size:(train_size + val_size)], x[-test_size:]

def install(package):
	subprocess.check_call([sys.executable, "-m", "pip", "install", package])

def print_sys(s):
	print(s, flush = True, file = sys.stderr)

def _parse_prop(search, proplist):
	"""Extract property value from record using the given urn search filter."""
	props = [i for i in proplist if all(item in i['urn'].items() for item in search.items())]
	if len(props) > 0:
		return props[0]['value'][list(props[0]['value'].keys())[0]]

def request(identifier, namespace='cid', domain='compound', operation=None, output='JSON', searchtype=None):
	"""
	copied from https://github.com/mcs07/PubChemPy/blob/e3c4f4a9b6120433e5cc3383464c7a79e9b2b86e/pubchempy.py#L238
	Construct API request from parameters and return the response.
	Full specification at http://pubchem.ncbi.nlm.nih.gov/pug_rest/PUG_REST.html
	"""
	API_BASE = 'https://pubchem.ncbi.nlm.nih.gov/rest/pug'
	text_types = str, bytes
	if not identifier:
		raise ValueError('identifier/cid cannot be None')
	# If identifier is a list, join with commas into string
	if isinstance(identifier, int):
		identifier = str(identifier)
	if not isinstance(identifier, text_types):
		identifier = ','.join(str(x) for x in identifier)

	# Build API URL
	urlid, postdata = None, None
	if namespace == 'sourceid':
		identifier = identifier.replace('/', '.')
	if namespace in ['listkey', 'formula', 'sourceid'] \
			or searchtype == 'xref' \
			or (searchtype and namespace == 'cid') or domain == 'sources':
		urlid = quote(identifier.encode('utf8'))
	else:
		postdata = urlencode([(namespace, identifier)]).encode('utf8')
	comps = filter(None, [API_BASE, domain, searchtype, namespace, urlid, operation, output])
	apiurl = '/'.join(comps)
	# Make request
	response = urlopen(apiurl, postdata)
	return response

def NegSample(df, column_names, frac, two_types):
	"""Negative Sampling for Binary Interaction Dataset

	Parameters
	----------
	df : pandas.DataFrame
		Data File
	column_names: list
		column names in the order of [id1, x1, id2, x2]
	"""
	x = int(len(df) * frac)
	id1, x1, id2, x2 = column_names
	df[id1] = df[id1].apply(lambda x: str(x))
	df[id2] = df[id2].apply(lambda x: str(x))

	if not two_types:
		df_unique = np.unique(df[[id1, id2]].values.reshape(-1))
		pos = df[[id1, id2]].values
		pos_set = set([tuple([i[0], i[1]]) for i in pos])
		np.random.seed(1234)
		samples = np.random.choice(df_unique, size=(x, 2), replace=True)
		neg_set = set([tuple([i[0], i[1]]) for i in samples if i[0] != i[1]]) - pos_set

		while len(neg_set) < x:
			sample = np.random.choice(df_unique, 2, replace=False)
			sample = tuple([sample[0], sample[1]])
			if sample not in pos_set:
				neg_set.add(sample)
		neg_list = [list(i) for i in neg_set]

		id2seq = dict(df[[id1, x1]].values)
		id2seq.update(df[[id2, x2]].values)

		neg_list_val = []
		for i in neg_list:
			neg_list_val.append([i[0], id2seq[i[0]], i[1], id2seq[i[1]], 0])

		df = df.append(pd.DataFrame(neg_list_val).rename(columns = {0: id1, 1: x1, 2: id2, 3: x2, 4: 'Y'})).reset_index(drop = True)
		return df
	else:
		df_unique_id1 = np.unique(df[id1].values.reshape(-1))
		df_unique_id2 = np.unique(df[id2].values.reshape(-1))

		pos = df[[id1, id2]].values
		pos_set = set([tuple([i[0], i[1]]) for i in pos])
		np.random.seed(1234)

		sample_id1 = np.random.choice(df_unique_id1, size=len(df), replace=True)
		sample_id2 = np.random.choice(df_unique_id2, size=len(df), replace=True)

		neg_set = set([tuple([sample_id1[i], sample_id2[i]]) for i in range(len(df)) if sample_id1[i] != sample_id2[i]]) - pos_set

		while len(neg_set) < len(df):
			sample_id1 = np.random.choice(df_unique_id1, size=1, replace=True)
			sample_id2 = np.random.choice(df_unique_id2, size=1, replace=True)

			sample = tuple([sample_id1[0], sample_id2[0]])
			if sample not in pos_set:
				neg_set.add(sample)
		neg_list = [list(i) for i in neg_set]

		id2seq1 = dict(df_temp[[id1, x1]].values)
		id2seq2 = dict(df_temp[[id2, x2]].values)

		neg_list_val = []
		for i in neg_list:
			neg_list_val.append([i[0], id2seq[i[0]], i[1], id2seq[i[1]], 0])

		df = df.append(pd.DataFrame(neg_list_val).rename(columns = {0: id1, 1: x1, 2: id2, 3: x2, 4: 'Y'})).reset_index(drop = True)
		return df

def uniprot2seq(ProteinID):
	"""Get protein sequence from Uniprot ID

	Parameters
	----------
	ProteinID : str
		Uniprot ID

	Returns
	-------
	str
		Amino acid sequence of input uniprot ID
	"""
	import urllib
	import string
	import urllib.request as ur

	ID = str(ProteinID)
	localfile = ur.urlopen('http://www.uniprot.org/uniprot/' + ID + '.fasta')
	temp = localfile.readlines()
	res = ''
	for i in range(1, len(temp)):
		res = res + temp[i].strip().decode("utf-8")
	return res

def cid2smiles(cid):
	try:
		smiles = _parse_prop({'label': 'SMILES', 'name': 'Canonical'}, json.loads(request(cid).read().decode())['PC_Compounds'][0]['props'])
	except:
		print('cid ' + str(cid) + ' failed, use NULL string')
		smiles = 'NULL'
	return smiles

def get_closet_match(predefined_tokens, test_token, threshold=0.8):
	"""Get the closest match by Levenshtein Distance.

	Parameters
	----------
	predefined_tokens : list of string
		Predefined string tokens.

	test_token : string
		User input that needs matching to existing tokens.

	threshold : float in (0, 1), optional (default=0.8)
		The lowest match score to raise errors.

	Returns
	-------

	"""
	prob_list = []

	for token in predefined_tokens:
		# print(token)
		prob_list.append(
			fuzz.ratio(str(token).lower(), str(test_token).lower()))

	assert (len(prob_list) == len(predefined_tokens))

	prob_max = np.nanmax(prob_list)
	token_max = predefined_tokens[np.nanargmax(prob_list)]

	# match similarity is low
	if prob_max / 100 < threshold:
		print_sys(predefined_tokens)
		raise ValueError(test_token,
						 "does not match to available values. "
						 "Please double check.")
	return token_max, prob_max / 100

def save_dict(path, obj):
	with open(path, 'wb') as f:
		pickle.dump(obj, f, pickle.HIGHEST_PROTOCOL)

def load_dict(path):
	with open(path, 'rb') as f:
		return pickle.load(f)

def retrieve_label_name_list(name):
	name = fuzzy_search(name, dataset_list)
	return dataset2target_lists[name]

def retrieve_dataset_names(name):
	return dataset_names[name]

def retrieve_all_benchmarks():
	return list(benchmark_names.keys())

def retrieve_benchmark_names(name):
	name = fuzzy_search(name, list(benchmark_names.keys()))
	datasets = benchmark_names[name]

	dataset_names = []

	for task, datasets in datasets.items():
		for dataset in datasets:
			dataset_names.append(dataset)
	return dataset_names

def to_submission_format(results):
	df = pd.DataFrame(results)
	def get_metric(x):
		metric = []
		for i in x:
			metric.append(list(i.values())[0])
		return [round(np.mean(metric), 3), round(np.std(metric), 3)]
	return dict(df.apply(get_metric, axis = 1))



import pandas as pd
import numpy as np
import sys
import warnings

warnings.filterwarnings("ignore")

from . import utils


class DataLoader:
    def __init__(self):
        pass

    def get_data(self, format='df'):
        '''
        Arguments:
            df: return pandas DataFrame; if not true, return np.arrays
        returns:
            self.drugs: drug smiles strings np.array
            self.targets: target Amino Acid Sequence np.array
            self.y: inter   action score np.array
        '''
        if format == 'df':
            return pd.DataFrame({self.entity1_name + '_ID': self.entity1_idx,
                                 self.entity1_name: self.entity1, 'Y': self.y})
        elif format == 'dict':
            return {self.entity1_name + '_ID': self.entity1_idx,
                    self.entity1_name: self.entity1, 'Y': self.y}
        elif format == 'DeepPurpose':
            return self.entity1, self.y
        elif format == 'sklearn':
            pass
        else:
            raise AttributeError("Please use the correct format input")

    def print_stats(self):
        print('There are ' + str(len(np.unique(
            self.entity1))) + ' unique ' + self.entity1_name.lower() + 's',
              flush=True, file=sys.stderr)

    def get_split(self, method='random', seed=42,
                  frac=[0.7, 0.1, 0.2]):
        '''
        Arguments:
            method: splitting schemes: random, cold_drug, cold_target
            seed: default 42
            frac: train/val/test split
        '''

        df = self.get_data(format='df')

        if method == 'random':
            return utils.create_fold(df, seed, frac)
        elif method == 'cold_' + self.entity1_name.lower():
            return utils.create_fold_setting_cold(df, seed, frac,
                                                  self.entity1_name)
        else:
            raise AttributeError("Please specify the correct splitting method")

    def label_distribution(self):
        utils.label_dist(self.y, self.name)

    def binarize(self, threshold=None, order='descending'):
        if threshold is None:
            raise AttributeError(
                "Please specify the threshold to binarize the data by "
                "'binarize(threshold = N)'!")

        if (len(np.unique(self.y)) == 2):
            print("The data is already binarized!", flush=True,
                  file=sys.stderr)
        else:
            print("Binariztion using threshold " + str(
                threshold) + ", default, we assume the smaller values are 1 "
                             "and larger ones is 0, you can change the order "
                             "by 'binarize(order = 'ascending')'",
                  flush=True, file=sys.stderr)
            if np.unique(self.y).reshape(-1, ).shape[0] < 2:
                raise AttributeError(
                    "Adjust your threshold, there is only one class.")
            self.y = utils.binarize(self.y, threshold, order)
        return self

    def __len__(self):
        return len(self.get_data(format='df'))

    def convert_to_log(self, form = 'standard'):
        print('To log space...', flush=True, file=sys.stderr)
        if form == 'binding':
            self.y = utils.convert_to_log(self.y)
        elif form == 'standard':
            self.sign = np.sign(self.y)
            self.y = self.sign * np.log(abs(self.y) + 1e-10)

    def convert_from_log(self, form = 'standard'):
        print('Convert Back To Original space...', flush=True, file=sys.stderr)
        if form == 'binding':
            self.y = utils.convert_back_log(self.y)
        elif form == 'standard':
            self.y = self.sign * (np.exp(self.sign * self.y) - 1e-10)

    def get_label_meaning(self, output_format='dict'):
        return utils.get_label_map(self.name, self.path, self.target,
                                   file_format=self.file_format,
                                   output_format=output_format)

    def balanced(self, oversample=False, seed=42):

        if len(np.unique(self.y)) > 2:
            raise AttributeError(
                "You should binarize the data first by calling "
                "data.binarize(threshold)",
                flush=True, file=sys.stderr)

        val = self.get_data()

        class_ = val.Y.value_counts().keys().values
        major_class = class_[0]
        minor_class = class_[1]

        if not oversample:
            print(
                " Subsample the majority class is used, if you want to do "
                "oversample the minority class, set 'balanced(oversample = True)'. ",
                flush=True, file=sys.stderr)
            val = pd.concat(
                [val[val.Y == major_class].sample(
                    n=len(val[val.Y == minor_class]), replace=False,
                    random_state=seed), val[val.Y == minor_class]]).sample(
                frac=1,
                replace=False,
                random_state=seed).reset_index(
                drop=True)
        else:
            print(" Oversample of minority class is used. ", flush=True,
                  file=sys.stderr)
            val = pd.concat(
                [val[val.Y == minor_class].sample(
                    n=len(val[val.Y == major_class]), replace=True,
                    random_state=seed), val[val.Y == major_class]]).sample(
                frac=1,
                replace=False,
                random_state=seed).reset_index(
                drop=True)
        return val



import pandas as pd
import numpy as np
import os, sys, json 
import warnings
warnings.filterwarnings("ignore")

from .utils import * 
from .metadata import download_oracle_names, oracle_names, distribution_oracles

class Oracle:
	def __init__(self, name, target_smiles = None, **kwargs):
		self.target_smiles = target_smiles
		self.kwargs = kwargs

		name = fuzzy_search(name, oracle_names)
		if name in download_oracle_names:
			self.name = oracle_load(name)
		else:
			self.name = name
		self.evaluator_func = None
		self.assign_evaluator() 

	def assign_evaluator(self):		
		if self.name == 'logp':			############################ molecular property 
			from .chem_utils import penalized_logp
			self.evaluator_func = penalized_logp 
		elif self.name == 'qed':
			from .chem_utils import qed
			self.evaluator_func = qed  
		elif self.name == 'drd2':
			from .chem_utils import drd2
			self.evaluator_func = drd2 
		elif self.name == 'cyp3a4_veith':
			from .chem_utils import cyp3a4_veith
			self.evaluator_func = cyp3a4_veith 
		elif self.name == 'sa':
			from .chem_utils import SA
			self.evaluator_func = SA 
		elif self.name == 'gsk3b':
			from .chem_utils import gsk3b
			oracle_object = gsk3b
			self.evaluator_func = oracle_object
		elif self.name == 'jnk3':
			from .chem_utils import jnk3
			oracle_object = jnk3()
			self.evaluator_func = oracle_object
		elif self.name == 'similarity_meta':	############################ oracle meta
			from .chem_utils import similarity_meta
			self.evaluator_func = similarity_meta(target_smiles = self.target_smiles, **self.kwargs)
		elif self.name == 'rediscovery_meta':
			from .chem_utils import rediscovery_meta 
			self.evaluator_func = rediscovery_meta(target_smiles = self.target_smiles, **self.kwargs)
		elif self.name == 'isomer_meta':
			from .chem_utils import isomer_meta 
			self.evaluator_func = isomer_meta(target_smiles = self.target_smiles, **self.kwargs)
		elif self.name == 'median_meta':
			from .chem_utils import median_meta 
			self.evaluator_func = median_meta(target_smiles_1 = self.target_smiles[0], 
											  target_smiles_2 = self.target_smiles[1], 
											  **self.kwargs) 
		elif self.name == 'rediscovery':	############################ guacamol 
			from .chem_utils import celecoxib_rediscovery, troglitazone_rediscovery, thiothixene_rediscovery
			self.evaluator_func = {"Celecoxib": celecoxib_rediscovery, 
								"Troglitazone": troglitazone_rediscovery, 
								"Thiothixene": thiothixene_rediscovery}
		elif self.name == 'celecoxib_rediscovery':
			from .chem_utils import celecoxib_rediscovery
			self.evaluator_func = celecoxib_rediscovery
		elif self.name == 'troglitazone_rediscovery':
			from .chem_utils import troglitazone_rediscovery
			self.evaluator_func = troglitazone_rediscovery
		elif self.name == 'thiothixene_rediscovery':
			from .chem_utils import thiothixene_rediscovery
			self.evaluator_func = thiothixene_rediscovery
		elif self.name == 'similarity':
			from .chem_utils import aripiprazole_similarity, albuterol_similarity, mestranol_similarity
			self.evaluator_func = {"Aripiprazole": aripiprazole_similarity,
									"Albuterol": albuterol_similarity,
									"Mestranol": mestranol_similarity}
		elif self.name == 'aripiprazole_similarity':
			from .chem_utils import aripiprazole_similarity
			self.evaluator_func = aripiprazole_similarity
		elif self.name == 'albuterol_similarity':
			from .chem_utils import albuterol_similarity
			self.evaluator_func = albuterol_similarity
		elif self.name == 'mestranol_similarity':
			from .chem_utils import mestranol_similarity
			self.evaluator_func = mestranol_similarity
		elif self.name == 'median':
			from .chem_utils import median1, median2
			self.evaluator_func = {'Median 1': median1,
									'Median 2': median2}
		elif self.name == 'median1':
			from .chem_utils import median1
			self.evaluator_func = median1
		elif self.name == 'median2':
			from .chem_utils import median2
			self.evaluator_func = median2
		elif self.name == 'mpo':
			from .chem_utils import osimertinib_mpo, fexofenadine_mpo, ranolazine_mpo, perindopril_mpo, amlodipine_mpo, sitagliptin_mpo, zaleplon_mpo
			self.evaluator_func = {'Osimertinib': osimertinib_mpo,
									'Fexofenadine': fexofenadine_mpo,
									'Ranolazine': ranolazine_mpo,
									'Perindopril': perindopril_mpo,
									'Amlodipine': amlodipine_mpo,
									'Sitagliptin': sitagliptin_mpo,
									'Zaleplon': zaleplon_mpo}
		elif self.name == 'osimertinib_mpo':
			from .chem_utils import osimertinib_mpo
			self.evaluator_func = osimertinib_mpo
		elif self.name == 'fexofenadine_mpo':
			from .chem_utils import fexofenadine_mpo
			self.evaluator_func = fexofenadine_mpo
		elif self.name == 'ranolazine_mpo':
			from .chem_utils import ranolazine_mpo
			self.evaluator_func = ranolazine_mpo
		elif self.name == 'perindopril_mpo':
			from .chem_utils import perindopril_mpo
			self.evaluator_func = perindopril_mpo
		elif self.name == 'amlodipine_mpo':
			from .chem_utils import amlodipine_mpo
			self.evaluator_func = amlodipine_mpo
		elif self.name == 'sitagliptin_mpo':
			from .chem_utils import sitagliptin_mpo
			self.evaluator_func = sitagliptin_mpo
		elif self.name == 'zaleplon_mpo':
			from .chem_utils import zaleplon_mpo
			self.evaluator_func = zaleplon_mpo
		elif self.name == 'valsartan_smarts':
			from .chem_utils import valsartan_smarts
			self.evaluator_func = valsartan_smarts
		elif self.name == 'hop':
			from .chem_utils import deco_hop, scaffold_hop
			self.evaluator_func = {'Deco Hop': deco_hop,
									'Scaffold Hop': scaffold_hop}
		elif self.name == 'deco_hop':
			from .chem_utils import deco_hop
			self.evaluator_func = deco_hop
		elif self.name == 'scaffold_hop':
			from .chem_utils import scaffold_hop
			self.evaluator_func = scaffold_hop
		elif self.name == 'isomers_c7h8n2o2':
			from .chem_utils import isomers_c7h8n2o2
			self.evaluator_func = isomers_c7h8n2o2
		elif self.name == 'isomers_c9h10n2o2pf2cl':
			from .chem_utils import isomers_c9h10n2o2pf2cl
			self.evaluator_func = isomers_c9h10n2o2pf2cl  
		elif self.name == 'isomers':
			from .chem_utils import isomers_c7h8n2o2, isomers_c9h10n2o2pf2cl
			self.evaluator_func = {'c7h8n2o2': isomers_c7h8n2o2,
									'c9h10n2o2pf2cl': isomers_c9h10n2o2pf2cl}
		elif self.name == 'askcos':  		#### synthetic analysis 
			from .chem_utils import askcos
			self.evaluator_func = askcos
		elif self.name == 'ibm_rxn':
			from .chem_utils import ibm_rxn
			self.evaluator_func = ibm_rxn
		elif self.name == 'molecule_one_synthesis':
			from .chem_utils import molecule_one_retro
			self.evaluator_func = molecule_one_retro(**self.kwargs)
		elif self.name == 'docking_score':
			from .chem_utils import docking_meta
			self.evaluator_func = docking_meta(**self.kwargs)
		# distribution oracle 
		# ['novelty', 'diversity', 'uniqueness', 'validity', 'fcd_distance', 'kl_divergence']  
		elif self.name == 'uniqueness':
			from .chem_utils import uniqueness
			self.evaluator_func = uniqueness 
			# uniqueness(list_of_smiles)
		elif self.name == 'validity':
			from .chem_utils import validity 
			self.evaluator_func = validity
			# def validity(list_of_smiles):
		elif self.name == 'diversity':
			from .chem_utils import diversity 
			self.evaluator_func = diversity 
			# diversity(list_of_smiles) 
		elif self.name == 'novelty':
			from .chem_utils import novelty
			self.evaluator_func = novelty
			# novelty(generated_smiles_lst, training_smiles_lst)
		elif self.name == 'fcd_distance':
			from .chem_utils import fcd_distance 
			self.evaluator_func = fcd_distance 
			# def fcd_distance(generated_smiles_lst, training_smiles_lst):
		elif self.name == 'kl_divergence':
			from .chem_utils import kl_divergence 
			self.evaluator_func = kl_divergence 
			# def kl_divergence(generated_smiles_lst, training_smiles_lst):

		else:
			return 

	def __call__(self, *args, **kwargs):
		if self.name in distribution_oracles:  
			return self.evaluator_func(*args, **kwargs)
			#### evaluator for distribution learning, e.g., diversity, validity   


		smiles_lst = args[0]
		if self.name == 'molecule_one_synthesis':
			return self.evaluator_func(*args, **kwargs)

		if type(smiles_lst) == list:

			#### evaluator for single molecule, 
			#### the input of __call__ is a single smiles OR list of smiles
			if isinstance(self.evaluator_func, dict):
				all_ = {}
				for i, fct in self.evaluator_func.items():
					results_lst = []
					for smiles in smiles_lst:
						results_lst.append(fct(smiles, *(args[1:]), **kwargs))
					all_[i] = results_lst
				return all_
			else:
				results_lst = []
				for smiles in smiles_lst:
					results_lst.append(self.evaluator_func(smiles, *(args[1:]), **kwargs))
				return results_lst
		else:	
			## a single smiles
			if type(self.evaluator_func) == dict:
				all_ = {}
				for i, fct in self.evaluator_func.items():
					all_[i] = fct(*args, **kwargs)
				return all_
			else:
				return self.evaluator_func(*args, **kwargs)







import warnings

warnings.filterwarnings("ignore")

from ..utils import *
from . import bi_pred_dataset, multi_pred_dataset
from ..metadata import dataset_names


class DTI(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["DTI"])
        self.entity1_name = 'Drug'
        self.entity2_name = 'Target'
        self.two_types = True

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)


class DDI(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["DDI"])
        self.entity1_name = 'Drug1'
        self.entity2_name = 'Drug2'
        self.two_types = False

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)

    def print_stats(self):
        print_sys('--- Dataset Statistics ---')
        print('There are ' + str(len(np.unique(
            self.entity1.tolist() + self.entity2.tolist()))) + ' unique drugs.',
              flush=True, file=sys.stderr)
        print('There are ' + str(len(self.y)) + ' drug-drug pairs.',
              flush=True, file=sys.stderr)
        print_sys('--------------------------')


class PPI(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["PPI"])
        self.entity1_name = 'Protein1'
        self.entity2_name = 'Protein2'
        self.two_types = False

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)

    def print_stats(self):
        print_sys('--- Dataset Statistics ---')
        print('There are ' + str(len(np.unique(
            self.entity1.tolist() + self.entity2.tolist()))) + ' unique proteins.',
              flush=True, file=sys.stderr)
        print('There are ' + str(len(self.y)) + ' protein-protein pairs.',
              flush=True, file=sys.stderr)
        print_sys('--------------------------')


class PeptideMHC(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["PeptideMHC"])
        self.entity1_name = 'Peptide'
        self.entity2_name = 'MHC'
        self.two_types = True

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)


class MTI(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["MTI"])
        self.entity1_name = 'miRNA'
        self.entity2_name = 'Target'
        self.two_types = True

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)


class GDA(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["GDA"])
        self.entity1_name = 'Gene'
        self.entity2_name = 'Disease'
        self.two_types = True

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)


class AntibodyAff(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["AntibodyAff"])
        self.entity1_name = 'Antibody'
        self.entity2_name = 'Antigen'
        self.two_types = True

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)


class DrugRes(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["DrugRes"])
        self.entity1_name = 'Drug'
        self.entity2_name = 'Cell Line'
        self.two_types = True

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)


class DrugSyn(multi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', print_stats=False):
        super().__init__(name, path, print_stats,
                         dataset_names=dataset_names["DrugSyn"])

        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)


class Catalyst(bi_pred_dataset.DataLoader):
    """Add a Class description here.

    Parameters
    ----------
    name :

    path :

    label_name :

    print_stats :

    """
    def __init__(self, name, path='./data', label_name=None,
                 print_stats=False):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["Catalyst"])
        self.entity1_name = 'Reactant'
        self.entity2_name = 'Product'
        self.two_types = True
        
        if print_stats:
            self.print_stats()

        print('Done!', flush=True, file=sys.stderr)



import pandas as pd
import numpy as np
import os, sys, json 
import warnings
warnings.filterwarnings("ignore")

from .. import base_dataset
from ..utils import *

class DataLoader(base_dataset.DataLoader):
	def __init__(self, name, path, label_name, print_stats, dataset_names, convert_format):
		if name.lower() in dataset2target_lists.keys():
			#print_sys("Tip: Use tdc.utils.retrieve_label_name_list('" + name.lower() + "') to retrieve all available label names.")
			if label_name is None:
				raise ValueError("Please select a label name. You can use tdc.utils.retrieve_label_name_list('" + name.lower() + "') to retrieve all available label names.")

		entity1, y, entity1_idx = property_dataset_load(name, path, label_name, dataset_names)
		
		self.entity1 = entity1
		self.y = y
		self.entity1_idx = entity1_idx
		self.name = name
		self.entity1_name = 'Drug'
		self.path = path
		self.file_format = 'csv'
		self.label_name = label_name
		self.convert_format = convert_format
		self.convert_result = None

	def get_data(self, format = 'df'):
		'''
		Arguments:
			df: return pandas DataFrame; if not true, return np.arrays			
		returns:
			self.drugs: drug smiles strings np.array
			self.targets: target Amino Acid Sequence np.array
			self.y: inter   action score np.array
		'''

		if (self.convert_format is not None) and (self.convert_result is None):
			from ..chem_utils import MolConvert
			converter = MolConvert(src = 'SMILES', dst = self.convert_format)
			convert_result = converter(self.entity1.values)
			self.convert_result = [i for i in convert_result]

		if format == 'df':
			if self.convert_format is not None:
				return pd.DataFrame({self.entity1_name + '_ID': self.entity1_idx, self.entity1_name: self.entity1, self.entity1_name + '_' + self.convert_format: self.convert_result, 'Y': self.y})
			else:
				return pd.DataFrame({self.entity1_name + '_ID': self.entity1_idx, self.entity1_name: self.entity1, 'Y': self.y})
			
		elif format == 'dict':
			if self.convert_format is not None:
				return {self.entity1_name + '_ID': self.entity1_idx.values, self.entity1_name: self.entity1.values, self.entity1_name + '_' + self.convert_format: self.convert_result, 'Y': self.y.values}
			else:
				return {self.entity1_name + '_ID': self.entity1_idx.values, self.entity1_name: self.entity1.values, 'Y': self.y.values}
		elif format == 'DeepPurpose':
			return self.entity1.values, self.y.values
		else:
			raise AttributeError("Please use the correct format input")

	def get_split(self, method = 'random', seed = 42, frac = [0.7, 0.1, 0.2]):
		'''
		Arguments:
			method: splitting schemes: random, cold_drug, scaffold split
			seed: default 42
			frac: train/val/test split
		'''

		df = self.get_data(format = 'df')

		if method == 'random':
			return create_fold(df, seed, frac)
		elif method == 'cold_' + self.entity1_name.lower():
			return create_fold_setting_cold(df, seed, frac, self.entity1_name)
		elif method == 'scaffold':
			return create_scaffold_split(df, seed, frac, self.entity1_name)
		else:
			raise AttributeError("Please specify the correct splitting method")

	def print_stats(self):
		print_sys('--- Dataset Statistics ---')
		try:
			x = np.unique(self.entity1)
		except:
			x = np.unique(self.entity1_idx)

		print(str(len(x)) + ' unique ' + self.entity1_name.lower() + 's.', flush = True, file = sys.stderr)
		print_sys('--------------------------')



import pandas as pd
import numpy as np
import os, sys, json 
import warnings
warnings.filterwarnings("ignore")

from .utils import *
from .metadata import get_task2category
from .evaluator import Evaluator

class BenchmarkGenerator:
	def __init__(self, name, path = './data'):
		'''
		-- PATH
			-- ADMET_Benchmark
				-- HIA_Hou
					-- train.csv
					-- valid.csv
					-- test.csv
				-- Caco2_Wang
					-- train.csv
					-- valid.csv
					-- test.csv
				....
		from tdc import BenchmarkGroup
		group = BenchmarkGroup(name = 'ADMET_Group', path = 'data/')
		predictions = {}

		for benchmark in group:
		   name = benchmark['name']
		   train, valid, test = benchmark['train'], benchmark['valid'], benchmark['test']
		   ## --- train your model --- ##
		   predictions[name] = y_pred

		group.evaluate(predictions)
		# {'caco2_wang': 0.234, 'hia_hou': 0.786}

		benchmark = group.get('Caco2_Wang')
		train, valid, test = benchmark['train'], benchmark['valid'], benchmark['test']
		## --- train your model --- ##
		group.evaluate(y_pred, benchmark = 'Caco2_Wang')
		# 0.234

		group.get_more_splits()
		'''
		
		task2category = get_task2category()
		name = fuzzy_search(name, retrieve_all_benchmarks())
		self.name = name
		self.datasets = retrieve_benchmark_names(name)

		# create a benchmark folder
		if not os.path.exists(path):
			os.mkdir(path)
		self.benchmark_path = os.path.join(path, name)
		if not os.path.exists(self.benchmark_path):
			os.mkdir(self.benchmark_path)

		self.dataloaders = {}
		# get data set local copy
		for task, datasets in self.datasets.items():
			category = task2category[task]
			# import the task
			exec('from .' + category + ' import ' + task)
			for dataset in datasets:
				print_sys('--- ' + dataset + ' ---')
				dataset_path = os.path.join(self.benchmark_path, dataset)

				if not os.path.exists(dataset_path):
					os.mkdir(dataset_path)
				data_ = eval(task + '(name = \'' + dataset + '\', path = \'' + path + '\')')
				
				self.dataloaders[dataset] = data_

				if not os.path.exists(os.path.join(dataset_path, 'train.csv')):
					
					out = data_.get_split(method = bm_split_names[self.name][dataset], seed = 42, frac = [0.7, 0.1, 0.2])
					
					out['train'].to_csv(os.path.join(dataset_path, 'train.csv'), index = False)
					out['valid'].to_csv(os.path.join(dataset_path, 'valid.csv'), index = False)
					out['test'].to_csv(os.path.join(dataset_path, 'test.csv'), index = False)
				else:
					print_sys('Local split is found for ' + dataset + '...')
		self.all_datasets = list(self.dataloaders.keys())

	def __iter__(self):
		self.index = 0
		self.num_datasets = len(self.all_datasets)
		return self

	def __next__(self):
		if self.index < self.num_datasets:
			dataset = self.all_datasets[self.index]
			print_sys('--- ' + dataset + ' ---')
			data_path = os.path.join(self.benchmark_path, dataset)
			train = pd.read_csv(os.path.join(data_path, 'train.csv'))
			valid = pd.read_csv(os.path.join(data_path, 'valid.csv'))
			test = pd.read_csv(os.path.join(data_path, 'test.csv'))
			self.index += 1
			return {'train': train, 'valid': valid, 'test': test, 'name': dataset}
		else:
			raise StopIteration
			


import pandas as pd
import numpy as np
import os, sys, json 
import warnings
warnings.filterwarnings("ignore")

from .utils import *
from .metadata import get_task2category, bm_metric_names, benchmark_names, bm_split_names
from .evaluator import Evaluator

class BenchmarkGroup:
	def __init__(self, name, path = './data', file_format='csv'):
		'''
		-- PATH
			-- ADMET_Benchmark
				-- HIA_Hou
					-- train_val.csv
					-- test.csv
				-- Caco2_Wang
					-- train_val.csv
					-- test.csv
				....
		from tdc import BenchmarkGroup
		group = BenchmarkGroup(name = 'ADMET_Group', path = 'data/')
		predictions = {}

		for benchmark in group:
		   name = benchmark['name']
		   train_val, test = benchmark['train_val'], benchmark['test']
		   
		   # to obtain any number of train:valid split
		   train, valid = group.get_train_valid_split(benchmark = name, split_type = 'default', seed = 42, frac = [0.875, 0.125])

		   ## --- train your model --- ##
		   predictions[name] = y_pred

		group.evaluate(predictions)
		# {'caco2_wang': 0.234, 'hia_hou': 0.786}

		benchmark = group.get('Caco2_Wang')
		train, valid, test = benchmark['train'], benchmark['valid'], benchmark['test']
		## --- train your model --- ##
		group.evaluate(y_pred, benchmark = 'Caco2_Wang')
		# 0.234
		
		from tdc import BenchmarkGroup
		group = BenchmarkGroup(name = 'ADMET_Group', path = 'data/')
		predictions_list = []

		for seed in [1, 2, 3, 4, 5]:
		    predictions = {}
		    for benchmark in group:
		        name = benchmark['name']
		        train_val, test = benchmark['train_val'], benchmark['test']
		        train, valid = group.get_train_valid_split(benchmark = name, split_type = 'default', seed = seed)
		        ## --- train your model --- ##
		        y_pred = [1] * len(test)
		        predictions[name] = y_pred
		    predictions_list.append(predictions)

		group.evaluate_many(predictions_list)

		'''
		
		self.name = bm_group_load(name, path)
		self.path = os.path.join(path, self.name)
		self.datasets = benchmark_names[self.name]
		self.dataset_names = []
		self.file_format = file_format

		for task, datasets in self.datasets.items():
			for dataset in datasets:
				self.dataset_names.append(dataset)


	def __iter__(self):
		self.index = 0
		self.num_datasets = len(self.dataset_names)
		return self

	def __next__(self):
		if self.index < self.num_datasets:
			dataset = self.dataset_names[self.index]
			print_sys('--- ' + dataset + ' ---')
			data_path = os.path.join(self.path, dataset)
			if self.file_format == 'csv':
				train = pd.read_csv(os.path.join(data_path, 'train_val.csv'))
				test = pd.read_csv(os.path.join(data_path, 'test.csv'))
			elif self.file_format == 'pkl':
				train = pd.read_pickle(os.path.join(data_path, 'train_val.pkl'))
				test = pd.read_pickle(os.path.join(data_path, 'test.pkl'))
			self.index += 1
			return {'train_val': train, 'test': test, 'name': dataset}
		else:
			raise StopIteration
			
	def get_train_valid_split(self, seed, benchmark, split_type = 'default'):
		print_sys('generating training, validation splits...')
		dataset = fuzzy_search(benchmark, self.dataset_names)
		data_path = os.path.join(self.path, dataset)
		if self.file_format == 'csv':
			train_val = pd.read_csv(os.path.join(data_path, 'train_val.csv'))
		elif self.file_format == 'pkl':
			train_val = pd.read_pickle(os.path.join(data_path, 'train_val.pkl'))

		if split_type == 'default':
			split_method = bm_split_names[self.name][dataset]
		else:
			split_method = split_type

		frac = [0.875, 0.125, 0.0]
		'''
		if len(frac) == 3:
			# train:val:test split
			train_frac = frac[0]/(frac[0] + frac[1])
			valid_frac = 1 - train_frac
			frac = [train_frac, valid_frac, 0.0]
		else:
			# train:val split
			frac = [frac[0], frac[1], 0.0]
		'''
		if split_method == 'scaffold':
			out = create_scaffold_split(train_val, seed, frac = frac, entity = 'Drug')
		elif split_method == 'random':
			out = create_fold(train_val, seed, frac = frac)
		elif split_method == 'combination':
			out = create_combination_split(train_val, seed, frac=frac)
		else:
			raise NotImplementedError
		return out['train'], out['valid']

	def get(self, benchmark):
		dataset = fuzzy_search(benchmark, self.dataset_names)
		data_path = os.path.join(self.path, dataset)
		if self.file_format == 'csv':
			train = pd.read_csv(os.path.join(data_path, 'train_val.csv'))
			test = pd.read_csv(os.path.join(data_path, 'test.csv'))
		elif self.file_format == 'pkl':
			train = pd.read_pickle(os.path.join(data_path, 'train_val.pkl'))
			test = pd.read_pickle(os.path.join(data_path, 'test.pkl'))
		return {'train_val': train, 'test': test, 'name': dataset}

	def evaluate(self, pred, true = None, benchmark = None):
		if true is None:
			# test set evaluation
			metric_dict = bm_metric_names[self.name]
			out = {}
			for data_name, pred_ in pred.items():
				data_name = fuzzy_search(data_name, self.dataset_names)
				data_path = os.path.join(self.path, data_name)
				if self.file_format == 'csv':
					test = pd.read_csv(os.path.join(data_path, 'test.csv'))
				elif self.file_format == 'pkl':
					test = pd.read_pickle(os.path.join(data_path, 'test.pkl'))
				y = test.Y.values
				evaluator = eval('Evaluator(name = \'' + metric_dict[data_name] + '\')')
				out[data_name] = {metric_dict[data_name]: round(evaluator(y, pred_), 3)}

				# If reporting accuracy across target classes
				if 'target_class' in test.columns:
					test['pred'] = pred_
					for c in test['target_class'].unique():
						data_name_subset = data_name + '_' + c
						test_subset = test[test['target_class']==c]
						y_subset = test_subset.Y.values
						pred_subset = test_subset.pred.values

						evaluator = eval('Evaluator(name = \'' +
									     metric_dict[data_name_subset] + '\')')
						out[data_name_subset] = {metric_dict[data_name_subset]:
							        round(evaluator(y_subset, pred_subset), 3)}
			return out
		else:
			# validation set evaluation
			if benchmark is None:
				raise ValueError('Please specify the benchmark name for us to retrieve the standard metric!')
			data_name = fuzzy_search(benchmark, self.dataset_names)
			metric_dict = bm_metric_names[self.name]
			evaluator = eval('Evaluator(name = \'' + metric_dict[data_name] + '\')')
			return {metric_dict[data_name]: round(evaluator(true, pred), 3)}

	def evaluate_many(self, preds):
		"""
		:param preds: list of dict<str dataset_name: list of float>
		:return: dict<dataset_name: [mean_metric_result, std_metric_result]

		This function returns the data in a format needed to submit to the Leaderboard
		"""
		if len(preds) < 5:
			return ValueError("Must have predictions from at least five runs for leaderboard submission")
		individual_results = []
		for pred in preds:
			retval = self.evaluate(pred)
			individual_results.append(retval)

		given_dataset_names = list(individual_results[0].keys())
		aggregated_results = {}
		for dataset_name in given_dataset_names:
			my_results = []
			for individual_result in individual_results:
				my_result = list(individual_result[dataset_name].values())[0]
				my_results.append(my_result)
			u = np.mean(my_results)
			std = np.std(my_results)
			aggregated_results[dataset_name] = [round(u, 3), round(std, 3)]
		return aggregated_results



import pandas as pd
import numpy as np
import os, sys, json 
import warnings
warnings.filterwarnings("ignore")

from ..utils import *
from . import generation_dataset
from ..metadata import dataset_names

class Reaction(generation_dataset.PairedDataLoader):

	def __init__(self, name, path = './data', print_stats = False, input_name = 'reactant', output_name = 'product'): 
		super().__init__(name, path, print_stats, input_name, output_name)

class RetroSyn(generation_dataset.PairedDataLoader):

	def __init__(self, name, path = './data', print_stats = False, input_name = 'product', output_name = 'reactant'): 
		super().__init__(name, path, print_stats, input_name, output_name)


class MolGen(generation_dataset.DataLoader):

	def __init__(self, name, path = './data', print_stats = False, column_name = 'smiles'): 
		super().__init__(name, path, print_stats, column_name)


class PairMolGen(generation_dataset.DataLoader):
	'''
		DataLoader for paired molecules 
		it contains 
			(1) input molecule X 
			(2) target molecule Y
			where Y is similar molecule with more desirable molecular property, 
			e.g., QED score, DRD score
		
	'''
	def __init__(self, name, path = './data', print_stats = False):
		if not os.path.exists(path):
			os.makedirs(path)
		# try:
		if True:
			name = name.strip().lower()
			X_lst, Y_lst = eval('molpair_process(name, path, dataset_names["PairMolGen"])')
			self.X_lst = X_lst 
			self.Y_lst = Y_lst 
			self.name = name
			if print_stats: 
				self.print_stats()
		# except:
		# 	raise AttributeError("Please use the correct and available dataset name!")
		print_sys('Done!')
	def get_data(self, format = 'df'):
		'''
			format: 

		'''

		if format == 'df':
			return pd.DataFrame({'X':self.X_lst, 'Y':self.Y_lst}) 
		elif format == 'dict':
			return {'X':self.X_lst, 'Y':self.Y_lst}
		else:
			return 

	def get_split(self, method = 'random', seed = 42, frac = [0.7, 0.1, 0.2]):
		'''
		Arguments:
			method: splitting schemes: random, cold_drug, scaffold split
			seed: set to 42
			frac: train/val/test split
		'''

		df = self.get_data(format = 'df')
		
		if method == 'random':
			return create_fold(df, seed, frac)
		else:
			raise AttributeError("Please specify the correct splitting method")



	def print_stats(self):
		print(self.name +  ' has ' + str(len(self.X_lst)) + ' molecule pairs.', flush = True, file = sys.stderr)




# class Reaction(generation_dataset.DataLoader):
# 	'''
# 		DataLoader for paired molecules 
# 		it contains 
# 			(1) input molecule X 
# 			(2) target molecule Y
# 			where Y is similar molecule with more desirable molecular property, 
# 			e.g., QED score, DRD score
		
# 	'''
# 	def __init__(self, name, path = './data', print_stats = True):
# 		if not os.path.exists(path):
# 			os.makedirs(path)
# 		try:
# 			name = name.strip().lower()
# 			assert name in ['uspto']
# 			reactant_lst, catalyst_lst, product_lst  = eval(name + '_process(name, path)')
# 			self.reactant_lst = reactant_lst 
# 			self.catalyst_lst = catalyst_lst
# 			self.product_lst = product_lst
# 			self.name = name
# 		except:
# 			raise AttributeError("Please use the correct and available dataset name!")

# 	def get_data(self, format = 'df'):
# 		'''
# 			format: 
				
# 		'''

# 		if format == 'df':
# 			return pd.DataFrame({'reactant':self.reactant_lst, 'product':self.product_lst, 'catalyst':self.catalyst_lst}) 
# 		elif format == 'dict':
# 			return {'reactant':self.reactant_lst, 'product':self.product_lst, 'catalyst':self.catalyst_lst}
# 		else:
# 			return 

# 	def get_split(self, method = 'random', seed = 'benchmark', frac = [0.7, 0.1, 0.2]):
# 		'''
# 		Arguments:
# 			method: splitting schemes: random, cold_drug, scaffold split
# 			seed: 'benchmark' seed set to 1234, or int values
# 			frac: train/val/test split
# 		'''

# 		if seed == 'benchmark':
# 			seed = 1234

# 		df = self.get_data(format = 'df')
		
# 		if method == 'random':
# 			return utils.create_fold(df, seed, frac)
# 		else:
# 			raise AttributeError("Please specify the correct splitting method")

# 	def catalyst_statistics(self):
# 		'''
# 			compute some statistics for catalyst, e.g., count/frequency, total num. 


# 				('', 418450)
# 				('C1COCC1', 98588)
# 				('C(Cl)Cl', 91790)
# 				('CN(C=O)C', 77304)
# 				('ClCCl', 64883)
# 				('CO', 64824)
# 				('O', 49760)
# 				('O1CCCC1', 45353)
# 				('C(O)C', 41220)
# 				('CN(C)C=O', 32733)
# 				('C(#N)C', 29184)
# 				('C1(C)C=CC=CC=1', 26739)
# 				('O1CCOCC1', 25727)
# 				('C(OCC)(=O)C', 16592)
# 				('CCO', 15147)
# 				('C(O)(=O)C', 13912)
# 				('CS(C)=O', 13893)
# 				('N1C=CC=CC=1', 13378)
# 				('CN(C=O)C.O', 13325)
# 				('CO.[Pd]', 13007)
# 		'''
# 		catalyst_num = len(set(self.catalyst_lst))
# 		print(str(catalyst_num) + " catalysts totally.")
# 		## 55595 catalysts totally. 
# 		from collections import defaultdict, Counter
# 		catalyst2cnt = defaultdict(lambda:0)
# 		for i in self.catalyst_lst:
# 			catalyst2cnt[i]+=1 
# 		for i in sorted(catalyst2cnt.items(), key=lambda x:x[1], reverse = True)[:20]:
# 			print(i)
# 		return 


# 	def print_stats(self):
# 		print(self.name +  ' has ' + str(len(self.reactant_lst)) + ' reactions.', flush = True, file = sys.stderr)








import pandas as pd
import numpy as np
import os, sys, json 
import warnings
warnings.filterwarnings("ignore")

from .. import base_dataset
from ..utils import *


class DataLoader(base_dataset.DataLoader):
	def __init__(self, name, path, print_stats, column_name):
		from ..utils import single_molecule_dataset_names 
		self.smiles_lst = distribution_dataset_load(name, path, single_molecule_dataset_names, column_name = column_name)  
		### including fuzzy-search 
		self.name = name 
		self.path = path 
		self.dataset_names = single_molecule_dataset_names
		if print_stats: 
			self.print_stats() 
		print_sys('Done!')
		
	def print_stats(self):
		print("There are " + str(len(self.smiles_lst)) + ' molecules ', flush = True, file = sys.stderr)

	def get_data(self, format = 'df'):
		if format == 'df':
			return pd.DataFrame({'smiles': self.smiles_lst})
		elif format == 'dict':
			return {'smiles': self.smiles_lst} 
		else:
			raise AttributeError("Please use the correct format input")

	def get_split(self, method = 'random', seed = 42, frac = [0.7, 0.1, 0.2]):
		'''
		Arguments:
			method: splitting schemes: random, cold_drug, cold_target
			seed: default 42
			frac: train/val/test split
		'''
		df = self.get_data(format = 'df')

		if method == 'random':
			return create_fold(df, seed, frac)
		else:
			raise AttributeError("Please use the correct split method")


class PairedDataLoader(base_dataset.DataLoader):
	def __init__(self, name, path, print_stats, input_name, output_name):
		'''
		Arguments:
			name: fuzzy name of the generation dataset. e.g., uspto50k, qed, drd, ... 
			path: directory path that stores the dataset, e.g., ./data
			print_stats: bool, whether print the stats.  
		returns:
			None
		'''
		from ..utils import paired_dataset_names 
		self.input_smiles_lst, self.output_smiles_lst = generation_paired_dataset_load(name, path, 
																					   paired_dataset_names, input_name, 
																					   output_name)  ### including fuzzy-search 
		self.name = name 
		self.path = path 
		self.dataset_names = paired_dataset_names
		if print_stats: 
			self.print_stats() 
		print_sys('Done!')
		
	def print_stats(self):
		print("There are " + str(len(self.input_smiles_lst)) + ' paired samples', flush = True, file = sys.stderr)


	def get_data(self, format = 'df'):
		if format == 'df':
			return pd.DataFrame({'input': self.input_smiles_lst, 'output':self.output_smiles_lst})
		elif format == 'dict':
			return {'input': self.input_smiles_lst, 'output':self.output_smiles_lst} 
		else:
			raise AttributeError("Please use the correct format input")


	def get_split(self, method = 'random', seed = 42, frac = [0.7, 0.1, 0.2]):
		'''
		Arguments:
			method: splitting schemes: random, cold_drug, cold_target
			seed: 42
			frac: train/val/test split
		'''

		df = self.get_data(format = 'df')

		if method == 'random':
			return create_fold(df, seed, frac)
		else:
			raise AttributeError("Please use the correct split method")

class DataLoader3D(base_dataset.DataLoader):
	### locally, unzip a folder, with the main file the dataframe with SMILES, Mol Object for various kinds of entities.
	### also, for each column, contains a sdf file. 

	def __init__(self, name, path, print_stats, dataset_names, column_name):
		self.df, self.path, self.name = three_dim_dataset_load(name, path, dataset_names)  
		if print_stats: 
			self.print_stats() 
		print_sys('Done!')
		
	def print_stats(self):
		print("There are " + str(len(self.df)) + ' data points ', flush = True, file = sys.stderr)

	def get_data(self, format = 'df', more_features = 'None'):
		
		if more_features in ['None', 'SMILES']:
			pass
		elif more_features in ['Graph3D', 'Coulumb', 'SELFIES']:
			try: 
				from rdkit.Chem.PandasTools import LoadSDF
				from rdkit import rdBase
				rdBase.DisableLog('rdApp.error')
			except:
				raise ImportError("Please install rdkit by 'conda install -c conda-forge rdkit'! ")	

			from ..chem_utils import MolConvert
			from ..metadata import sdf_file_names

			convert = MolConvert(src = 'SDF', dst = more_features)
			for i in sdf_file_names[self.name]:
				self.df[i + '_' + more_features] = convert(self.path + i + '.sdf')

		if format == 'df':
			return self.df
		else:
			raise AttributeError("Please use the correct format input")

	def get_split(self, method = 'random', seed = 42, frac = [0.7, 0.1, 0.2]):
		'''
		Arguments:
			method: splitting schemes: random, cold_drug, cold_target
			seed: default 42
			frac: train/val/test split
		'''
		df = self.get_data(format = 'df')

		if method == 'random':
			return create_fold(df, seed, frac)
		else:
			raise AttributeError("Please use the correct split method")





import pandas as pd
import numpy as np
import os, sys, json 
import warnings
warnings.filterwarnings("ignore")

from .. import base_dataset
from ..utils import *

class DataLoader(base_dataset.DataLoader):
	def __init__(self, name, path, print_stats, dataset_names):
		if name.lower() in dataset2target_lists.keys():
			if label_name is None:
				raise ValueError("Please select a label name. You can use tdc.utils.retrieve_label_name_list('" + name.lower() + "') to retrieve all available label names.")

		df = multi_dataset_load(name, path, dataset_names)

		self.df = df
		self.name = name
		self.path = path

	def get_data(self, format = 'df'):
		if format == 'df':
			return self.df
		elif format == 'dict':
			return dict(self.df)
		else:
			raise AttributeError("Please use the correct format input")

	def print_stats(self):
		print_sys('--- Dataset Statistics ---')
		print(str(len(self.df)) + ' data points.', flush = True, file = sys.stderr)
		print_sys('--------------------------')

	def get_split(self, method = 'random', seed = 42, frac = [0.7, 0.1, 0.2], column_name = None):

		df = self.get_data(format = 'df')

		if method == 'random':
			return create_fold(df, seed, frac)
		if (column_name is not None) and (column_name in df.columns.values): 
			if method == 'cold_split':		
				return create_fold_setting_cold(df, seed, frac, column_name)
		elif method == 'combination':
			return create_combination_split(df, seed, frac)
		else:
			raise AttributeError("Please select from random_split, or cold_split, if cold split. please specify the column name!")


# -*- coding: utf-8 -*-
"""Docstring to be finished.
"""
# Author: TDC Team
# License: MIT

import sys

import warnings

warnings.filterwarnings("ignore")

from . import single_pred_dataset
from ..utils import print_sys
from ..utils import train_val_test_split
from ..metadata import dataset_names


class ADME(single_pred_dataset.DataLoader):
    """Docstring to be finished.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["ADME"], convert_format = convert_format)
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)

class Tox(single_pred_dataset.DataLoader):
    """Docstring to be finished.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["Toxicity"], convert_format = convert_format)
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)

class Epitope(single_pred_dataset.DataLoader):
    """Docstring to be finished.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["Epitope"], convert_format = convert_format)
        self.entity1_name = 'Antigen'
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)

class Paratope(single_pred_dataset.DataLoader):
    """Docstring to be finished.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["Paratope"], convert_format = convert_format)
        self.entity1_name = 'Antibody'
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)

class HTS(single_pred_dataset.DataLoader):
    """Docstring to be finished.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["HTS"], convert_format = convert_format)
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)

class Develop(single_pred_dataset.DataLoader):
    """Docstring to be finished.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["Develop"], convert_format = convert_format)
        self.entity1_name = 'Antibody'
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)

class QM(single_pred_dataset.DataLoader):
    """Docstring to be finished.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["QM"], convert_format = convert_format)
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)

class Yields(single_pred_dataset.DataLoader):
    """Docstring to be finished.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["Yields"], convert_format = convert_format)
        self.entity1_name = 'Reaction'
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)

class CRISPROutcome(single_pred_dataset.DataLoader):
    """DNA repair outcomes following a CRISPR experiment.

    Parameters
    ----------
    name : str
        Description of the variable.

    path : str, optional (default="data")
        Description of the variable.

    label_name : str, optional (default=None)
        Description of the variable.

    print_stats : bool, optional (default=True)
        Description of the variable.
    """

    def __init__(self, name, path='./data', label_name=None, print_stats=False, convert_format=None):
        super().__init__(name, path, label_name, print_stats,
                         dataset_names=dataset_names["CRISPROutcome"], convert_format = convert_format)
        self.entity1_name = 'GuideSeq'
        if print_stats:
            self.print_stats()
        print('Done!', flush = True, file = sys.stderr)
